part of 'customer_change_bloc.dart';

@immutable
abstract class CustomerChangeState extends Equatable {
  const CustomerChangeState();
  @override
  List<Object> get props => [];
}

class CustomerChangeInitial extends CustomerChangeState {
  const CustomerChangeInitial();
}

class CustomerChangeLoading extends CustomerChangeState {
  const CustomerChangeLoading();
}

class CustomerSaving extends CustomerChangeState {
  const CustomerSaving();
}

class CustomerDataChanged extends CustomerChangeState {
  const CustomerDataChanged();
}

class CustomerSaveSuccess extends CustomerChangeState {
  const CustomerSaveSuccess();
}

class CustomerSaveError extends CustomerChangeState {
  final String message;
  const CustomerSaveError({required this.message});

  @override
  List<Object> get props => [message];
}
